#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""ADS-B Cursor-on-Target Constants."""

import logging
import os

__author__ = "Greg Albrecht W2GMD <oss@undef.net>"
__copyright__ = "Copyright 2022 Greg Albrecht"
__license__ = "Apache License, Version 2.0"


if os.getenv("DEBUG"):
    LOG_LEVEL = logging.DEBUG
    LOG_FORMAT = logging.Formatter(
        (
            "%(asctime)s adsbcot %(levelname)s %(name)s.%(funcName)s:%(lineno)d "
            " - %(message)s"
        )
    )
    logging.debug("adsbcot Debugging Enabled via DEBUG Environment Variable.")
else:
    LOG_LEVEL = logging.INFO
    LOG_FORMAT = logging.Formatter(("%(asctime)s adsbcot %(levelname)s - %(message)s"))

DEFAULT_POLL_INTERVAL: int = 30
DEFAULT_COT_STALE: int = 120
DEFAULT_DUMP1090_TCP_RAW_PORT: str = 30002
DEFAULT_DUMP1090_TCP_BEAST_PORT: str = 30005
